# coffeelint: disable=max_line_length

shell = require("shelljs")
ko = require "knockout"
debug = require("debug")("omg-testcall")
Producer = require "../../ProducerClass"
call = require './call'
logs = require './logs'

testcallFolder = "/tmp/testcall"

#create testcall folder where options and logs will be stored
shell.mkdir testcallFolder if not shell.test "-e", testcallFolder
#copy all files that need write access to new testcall folder
shell.cp "-f", __dirname + "/files/*", "#{testcallFolder}/"

class Testcall extends Producer
  constructor: ->
    super "testcall"

    # pass on all emitted events from logs (e.g. "subscribeTo" is especially important here)
    logsEmit = logs.emit
    logs.emit = (args...) =>
      @emit.apply @, args
      logsEmit.apply logs, args

    @sendStatus "stopped"
    @gatelogsRunning = logs.running
    @callIsWaiting = false

    call.running.subscribe (val) ->
      unless val
        logs.stop()

    @gatelogsOrCallRunning = ko.computed
      read: =>
        if @gatelogsRunning() or call.running()
          return true
        else
          return false
      deferEvaluation: true

    ko.computed =>
      if call.running()
        debug "Call is running"
        @sendStatus "running"
      else
        debug "Call is not running"
        @sendStatus "stopped"

      unless @gatelogsOrCallRunning()
        debug "Log and call not running"
        @sendStatus "stopped"

    logs.on "data", (data) =>
      debug "parsed data from gatelogs", data
      @sendData data

    call.on "finished", ->
      debug "call finished, stopping logs"
      logs.stop()


  start: (callObject) =>
    debug "starting call"
    if @gatelogsOrCallRunning()

      #set dirty flag so not more than one calls can be waiting
      #if there is already another call waiting, don't proceed
      if @callIsWaiting
        return
      else
        @callIsWaiting = true

      # wait for gatelogs and call to stop, then try again
      retryWhenAllStopped = @gatelogsOrCallRunning.subscribe (stillRunning) =>
        unless stillRunning
          debug "Previous call and gatelogs stopped. Running new call"
          retryWhenAllStopped.dispose()
          @start callObject
          @callIsWaiting = false

      @sendData ["Waiting for previous call to finish."]
      debug "Waiting for previous call to finish."
      logs.stop()

      # don't proceed with new call
      return

    @sendStatus "started"

    if @gatelogsRunning()
      debug "Gatelogs are running already, starting call"
      call.start(callObject)
    else
      debug "Gatelogs are not running, yet."
      startWhenLogsAreRunning = @gatelogsRunning.subscribe (val) ->
        debug "Gatelogs status changed"
        if val
          debug "Gatelogs are running now, starting call"
          startWhenLogsAreRunning.dispose()
          call.start(callObject)

    debug "starting Gatelogs"
    logs.start()

  stop: () ->
    logs.stop()
    call.stop()

module.exports = new Testcall()
